<?php
/**
 * Nic_Status_SQL:: SQL implementing status storage backend for Nic.
 *
 * $Horde: nic/lib/Status/sql.php,v 1.8 2004/10/31 10:32:55 jan Exp $
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @version $Revision: 1.8 $
 * @since   Network 0.1
 * @package Nic
 */
class Nic_Status_SQL extends Nic_Status {

    /** Hash containing connection parameters. */
    var $_params = array();

    /** Handle for the current database connection.
        @var object DB $db */
    var $_db;

    /** Boolean indicating whether or not we're connected to the SQL server. */
    var $_connected = false;

    /**
     * Constructs a new SQL storage object.
     *
     * @param string $device    Device ID to store / retrive values for.
     * @param array  $params    A hash containing connection parameters.
     */
    function Nic_Status_SQL($params)
    {
        $this->_params = $params;
    }

    /**
     * Retrieve the current status for a subsystem of a hostname.
     *
     * @param string $hostname The hostname of the device.
     * @param string $subsystem The subsystem on the device.
     *
     * @return mixed    (constant) NIC_STATUS_* constant or PEAR_Error on failure.
     */
    function retrieveCurrentStatus($hostname, $subsystem)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT status_comment, status_status, status_time FROM %s WHERE status_hostname = %s AND status_subsystem = %s ORDER BY status_time DESC LIMIT 0, 1',
                         $this->_params['table'], $this->_db->quote($hostname), $this->_db->quote($subsystem));

        /* Execute the query. */
        $result = $this->_db->query($query);

        $status = array();
        $status['time'] = time();
        $status['status'] = NIC_STATUS_UNKNOWN;
        $status['comment'] = '';

        if (isset($result) && !is_a($result, 'PEAR_ERROR')) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (is_a($row, 'PEAR_Error') || !is_array($row)) {
                return NIC_STATUS_UNKNOWN;
            }

            $status['time'] = $row['status_time'];
            $status['status'] = $row['status_status'];
            $status['comment'] = $row['status_comment'];
        }

        return $status;
    }

    /**
     * TODO
     */
    function retrieveStatusHistory($hostname, $subsystem, $starttime, $endtime = null)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        if (is_null($endtime)) {
            $endtime = time();
        }

        /* Build the SQL query. */
        $query = sprintf('SELECT status_comment, status_status, status_time FROM %s WHERE status_hostname = %s AND status_subsystem = %s AND status_time >= %d AND status_time <= %d ORDER BY status_time',
                         $this->_params['table'], $this->_db->quote($hostname), $this->_db->quote($subsystem), $starttime, $endtime);

        /* Execute the query. */
        $result = $this->_db->query($query);

        $status = array();
        if (isset($result) && !is_a($result, 'PEAR_Error')) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (is_a($row, 'PEAR_Error')) {
                return $subsystems;
            }

            while ($row && !is_a($row, 'PEAR_Error')) {
                $status[$row['status_time']]['status'] = $row['status_status'];
                $status[$row['status_time']]['comment'] = $row['status_comment'];

                /* Advance to the new row in the result set. */
                $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            }
        }
        return $status;
    }

    /**
     * Retrieve all the subsystems that have a status stored for them.
     *
     * @param array $hostnames (optional) Only return the subsytems that belong to
     *                         these hostnames.
     * @param int $lastactive (optional) Only return hostnames that have data
     *                        recorded since this timestamp.
     *
     * @return array Array of stings of subsystems.
     */
    function retrieveSubsystems($hostnames = null, $lastactive = null)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT DISTINCT status_subsystem FROM %s', $this->_params['table']);

        if (is_array($hostnames) && count($hostnames) > 0) {
            $query .= " WHERE ( status_hostname = " . $this->_db->quote($hostnames[0]);
            $i = 1;
            while ($i< count($hostnames)) {
                $query .= " OR status_hostname = " . $this->_db->quote($hostnames[$i]);
                $i ++;
            }
            $query .= ")";

            if (!is_null($lastactive)) {
                $query .= ' AND status_time >= ' . $this->_db->quote($lastactive);
            }
        } else {
            if (!is_null($lastactive)) {
                $query .= ' WHERE status_time >= ' . $this->_db->quote($lastactive);
            }
        }

        /* Execute the query. */
        $result = $this->_db->query($query);

        $subsystems = array();

        if (isset($result) && !is_a($result, 'PEAR_Error')) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (is_a($row, 'PEAR_Error')) {
                return $subsystems;
            }

            while ($row && !is_a($row, 'PEAR_Error')) {
                $subsystems[] = $row['status_subsystem'];

                /* Advance to the new row in the result set. */
                $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            }
        }

        sort($subsystems);
        return $subsystems;
    }

    /**
     * Retrieve all the hostnames that have a status stored for them.
     *
     * @param int $lastactive (optional) Only return hostnames that have data
     *                        recorded since this timestamp.
     *
     * @return array Array of stings of hostnames
     */
    function retrieveHostnames($lastactive = null)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT DISTINCT status_hostname FROM %s',
                         $this->_params['table']);

        if (!is_null($lastactive)) {
            $query .= ' WHERE status_time >= ' . $this->_db->quote($lastactive);
        }

        /* Execute the query. */
        $result = $this->_db->query($query);

        $hostnames = array();

        if (isset($result) && !is_a($result, 'PEAR_Error')) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (is_a($row, 'PEAR_Error')) {
                return $hostnames;
            }

            while ($row && !is_a($row, 'PEAR_Error')) {
                $hostnames[] = $row['status_hostname'];

                /* Advance to the new row in the result set. */
                $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            }
        }

        sort($hostnames);
        return $hostnames;
    }

    /**
     * Add a new data value to the storage system.
     *
     * @param string    $name       The name of the counter to retrieve.
     * @param counstant $dataType   The type of data being retrieved.
     * @param int       $time       The time the reading was taken.
     * @param int       $value      The value to add.
     *
     * @return mixed    (boolean) true on success, PEAR_Error on failure.
     */
    function setStatus($time, $hostname, $subsystem, $status, $comment = '')
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        $query = "INSERT INTO " . $this->_params['table'] . "(status_time, status_hostname, status_subsystem, status_status, status_comment)";
        $query .= sprintf("VALUES (%d, %s, %s, %s, %s)", $time, $this->_db->quote($hostname), $this->_db->quote($subsystem), $status, $this->_db->quote($comment));

        /* Attempt the insertion query. */
        $result = $this->_db->query($query);

        /* Return an error immediately if the query failed. */
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        return true;
    }

    /**
     * Remove old status records from database.
     *
     * @param int        $removebefore   Remove older records
     *
     * @return mixed     (boolean) true on success, PEAR_Error on failure.
     */
    function purgeStatus($removebefore)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();
                                                                                                                                                             
        $query = "DELETE FROM " . $this->_params['table'] . ' WHERE status_time < ' . $removebefore;
                                                                                                                                                             
        $result = $this->_db->query($query);

        /* Return an error immediately if the query failed. */
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
                                                                                                                                                             
        return true;
    }


    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean    True on success; exits (Horde::fatal()) on error.
     */
    function _connect()
    {
        if (!$this->_connected) {
            Horde::assertDriverConfig($this->_params, 'status',
                array('phptype', 'hostspec', 'username', 'database', 'table'),
                'nic status SQL');

            /* Connect to the SQL server using the supplied parameters. */
            require_once 'DB.php';
            $this->_db = &DB::connect($this->_params,
                                     array('persistent' => !empty($this->_params['persistent'])));
            if (is_a($this->_db, 'PEAR_Error')) {
                Horde::fatal($this->_db, __FILE__, __LINE__);
            }

            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

            $this->_connected = true;
        }

        return true;
    }

}
